#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
from bs4 import Tag, NavigableString, Comment

def copy(element):
	"""
	beautifulsoup4 객체의 요소를 복제합니다.
	
	오직 해당 요소의 정보만 복사합니다. 자식들에 대한 정보는 원본과 공유됩니다.
	
	.. bugs::
		work around bug where there is no builder set https://bugs.launchpad.net/beautifulsoup/+bug/1307471.
	
	.. 이 함수 작성에 다음 문서를 참조하였음.
		http://stackoverflow.com/questions/23057631/clone-element-with-beautifulsoup
	
	:param element: 복제 할 원소.
	:type element: bs4.Tag, bs4.NavigableString, bs4.Comment
	:return: 복제된 원소.
	"""
	if isinstance(element, (NavigableString, Comment)):
		return type(element)(element)
	
	clone_element = Tag(None, element.builder, element.name, element.namespace, element.nsprefix)
	clone_element.attrs = dict(element.attrs)
	for attr in ('can_be_empty_element', 'hidden'):
		setattr(clone_element, attr, getattr(element, attr))
	for child in element.contents:
		clone_element.append(child)
	return clone_element

def deepcopy(element):
	"""
	beautifulsoup4 객체의 요소를 재귀적으로 복제합니다.
	
	요소가 가진 정보 및 자식의 모든 정보를 재귀적으로 복제합니다.
	
	이 함수를 통하여, beautifulsoup4 4.0.2에서 append 함수 사용시 발생하는 DOM 깨짐현상(기존에 존재하던 태그에 접근 불가능해지거나, 각 메소드들 별로 원래 정상적으로 접근하게될 결과의 일부분만 얻게 되는 현상)을 피할 수 있습니다.
	
	.. 이 함수 작성에 다음 문서를 참조하였음.
		http://stackoverflow.com/questions/23057631/clone-element-with-beautifulsoup
	
	:param element: 복제 할 원소.
	:type element: bs4.Tag, bs4.NavigableString, bs4.Comment
	:return: 복제된 원소.
	"""
	if isinstance(element, (NavigableString, Comment)):
		return type(element)(element)
	
	clone_element = Tag(None, element.builder, element.name, element.namespace, element.nsprefix)
	# work around bug where there is no builder set
	# https://bugs.launchpad.net/beautifulsoup/+bug/1307471
	clone_element.attrs = dict(element.attrs)
	for attr in ('can_be_empty_element', 'hidden'):
		setattr(clone_element, attr, getattr(element, attr))
	for child in element.contents:
		clone_element.append(deepcopy(child))
	return clone_element



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function

from . import color

#호환 설정
from ... import terminal
terminal.color = color



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function

def urshift(val, n):
	"""
	부호 없이 식을 비트 단위로 오른쪽으로 이동합니다.
	
	자바스크립트의 >>> 연산자와 같은 역할을 합니다. 예를 들어 '(-1000) >>> 3'는 'ufp.math.rshift(-1000, 3)'와 같습니다.
	
	.. 참조 : http://stackoverflow.com/questions/5832982/how-to-get-the-logical-right-binary-shift-in-python
	
	:param val: 식
	:param n: 비트 수
	:return: 부호 없이 식을 비트 단위로 오른쪽으로 이동한 결과 값.
	"""
	if val >= 0:
		return val >> n
	else:
		return ( val + 0x100000000 ) >> n
	pass



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function

from ..repr import make_object as make_repr

#호환 설정
from .. import ufp as b
b.make_repr = make_repr
b.__all__.append('make_repr')



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function

RGB_MIN_VALUE = 0
RGB_MAX_VALUE = 255

GRAYSCALE_MIN_VALUE = 0
GRAYSCALE_MAX_VALUE = 255

# 호환 설정
from .. import image
image.RGB_MIN_VALUE = RGB_MIN_VALUE
image.RGB_MAX_VALUE = RGB_MAX_VALUE
image.GRAYSCALE_MIN_VALUE = GRAYSCALE_MIN_VALUE
image.GRAYSCALE_MAX_VALUE = GRAYSCALE_MAX_VALUE



#!/usr/bin/env python
#-*- coding: utf-8 -*-

from __future__ import unicode_literals, absolute_import, division, print_function
import itertools

def _get_instance_name(instance):
	if isinstance(instance, unicode):
		return instance
	if isinstance(instance, object):
		return type(instance).__name__
	return instance.__class__.__name__

def make_type(type_name, class_name):
	"""
	형식 repr 문자열을 생성합니다.
	
	:param type_name: 형식명.
	:type type_name: unicode
	:param class_name: 클래스 명.
	:type class_name: unicode, instance
	:return: u"<{type_name} '{class_name}'>"
	:rtype: unicode
	"""
	return "<{type_name} '{class_name}'>".format(
		type_name = type_name,
		class_name = _get_instance_name(class_name)
	)

def make_object(type_name, values=[], orderd_items=[], items={}, **kwargs):
	"""
	객체 repr 문자열을 생성합니다.
	
	:param type_name: 형식명. instance를 줄 경우, 해당 오브젝트의 클래스명을 사용합니다.
	:type type_name: unicode, instance
	:param values: 값들. values, args순서로 배치됩니다.
	:type values: [value, ...]
	:param items: 값들. orderd_items, items, kwargs순서로 배치됩니다.
	:type items: {key:value, ...}
	:param orderd_items: 값들. 전달된 순서대로 배치합니다.
	:type orderd_items: [(key,value), ...]
	:param **kwargs: 값들.\n
		형식과 값은 items의 각 요소와 같습니다.
	:return: u'{type_name}({value}, ..., {key}={value}, ...)'
	:rtype: unicode
	"""
	#형식명 설정
	if isinstance(type_name, unicode):
		pass
	elif isinstance(type_name, object):
		type_name = type(type_name).__name__
	else:
		type_name = type_name.__class__.__name__
	
	#값 목록 작성
	def convert(value):
		if isinstance(value, unicode):
			buffer = value.replace("'", r"\'")
			return "'{0}'".format(buffer)
		return repr(value)
	repr_list = list()
	for value in values:
		buffer = convert(value)
		repr_list.append(buffer)
	for key, value in itertools.chain(orderd_items, items.items(), kwargs.items()):
		buffer = convert(value)
		buffer = '{key}={value}'.format(key=key, value=buffer)
		repr_list.append(buffer)
	
	return '{type_name}({repr})'.format(
		type_name = _get_instance_name(type_name), 
		repr = ', '.join(repr_list)
	)
